#include <stdio.h>
#include <stdlib.h>
#include "chess.h"
#include "data.h"

/* last modified 03/11/98 */
/*
********************************************************************************
*                                                                              *
*   History_*() is used to maintain the history database.  this is a set of    *
*   counts, indexed by the 12-bit value [to,from], that contains the relative  *
*   effectiveness of this move as a refutation.  this effectiveness is simply  *
*   an arbitrary value that varies significantly and inversely to the depth of *
*   the sub-tree refuted by each move.                                         *
*                                                                              *
*   these routines also maintain the killer move lists as well, since they are *
*   similar in nature.                                                         *
*                                                                              *
********************************************************************************
*/
void HistoryBest(TREE *tree, int ply, int depth, int wtm)
{
  register int index, temp;
/*
 ----------------------------------------------------------
|                                                          |
|   if the best move so far is a capture or a promotion,   |
|   return, since we try good captures and promotions      |
|   before searching history heuristic moves anyway.       |
|                                                          |
 ----------------------------------------------------------
*/
  if (CaptureOrPromote(tree->pv[ply].path[ply])) return;
/*
 ----------------------------------------------------------
|                                                          |
|   otherwise, use the [to,from] as an index and increment |
|   the appropriate history table/entry.                   |
|                                                          |
 ----------------------------------------------------------
*/
  index=tree->pv[ply].path[ply] & 4095;
  if (wtm) history_w[index]+=depth*depth;
  else history_b[index]+=depth*depth;
/*
 ----------------------------------------------------------
|                                                          |
|   now, add the same move to the current killer moves if  |
|   it is not already there, otherwise, increment the      |
|   appropriate counter and sort if needed.                |
|                                                          |
 ----------------------------------------------------------
*/
  if (tree->killer_move1[ply] == tree->pv[ply].path[ply]) tree->killer_count1[ply]++;
  else if (tree->killer_move2[ply] == tree->pv[ply].path[ply]) {
    tree->killer_count2[ply]++;
    if (tree->killer_count1[ply] < tree->killer_count2[ply]) {
      temp=tree->killer_count1[ply];
      tree->killer_count1[ply]=tree->killer_count2[ply];
      tree->killer_count2[ply]=temp;
      temp=tree->killer_move1[ply];
      tree->killer_move1[ply]=tree->killer_move2[ply];
      tree->killer_move2[ply]=temp;
    }
  }
  else {
    tree->killer_count2[ply]=1;
    tree->killer_move2[ply]=tree->pv[ply].path[ply];
  }
}

void HistoryRefutation(TREE *tree, int ply, int depth, int wtm)
{
  register int index, temp;
/*
 ----------------------------------------------------------
|                                                          |
|   if the best move so far is a capture or a promotion,   |
|   return, since we try good captures and promotions      |
|   before searching history heuristic moves anyway.       |
|                                                          |
 ----------------------------------------------------------
*/
  if (CaptureOrPromote(tree->current_move[ply])) return;
/*
 ----------------------------------------------------------
|                                                          |
|   otherwise, use the [to,from] as an index and increment |
|   the appropriate history table/entry.                   |
|                                                          |
 ----------------------------------------------------------
*/
  index=tree->current_move[ply] & 4095;
  if (wtm) history_w[index]+=depth*depth;
  else history_b[index]+=depth*depth;
/*
 ----------------------------------------------------------
|                                                          |
|   now, add the same move to the current killer moves if  |
|   it is not already there, otherwise, increment the      |
|   appropriate counter and sort if needed.                |
|                                                          |
 ----------------------------------------------------------
*/
  if (tree->killer_move1[ply] == tree->current_move[ply]) tree->killer_count1[ply]++;
  else if (tree->killer_move2[ply] == tree->current_move[ply]) {
    tree->killer_count2[ply]++;
    if (tree->killer_count1[ply] < tree->killer_count2[ply]) {
      temp=tree->killer_count1[ply];
      tree->killer_count1[ply]=tree->killer_count2[ply];
      tree->killer_count2[ply]=temp;
      temp=tree->killer_move1[ply];
      tree->killer_move1[ply]=tree->killer_move2[ply];
      tree->killer_move2[ply]=temp;
    }
  }
  else {
    tree->killer_count2[ply]=1;
    tree->killer_move2[ply]=tree->current_move[ply];
  }
}
